/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#ifndef __LIPS_MSG_FOLDER_H
#define __LIPS_MSG_FOLDER_H


#include <msgmessage.h>


#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */


#define	MSG_FOLDER_ROOT			0
#define	MSG_FOLDER_SMS_ROOT		1
#define	MSG_FOLDER_MMS_ROOT		2
#define	MSG_FOLDER_EMS_ROOT		3
#define	MSG_FOLDER_EMAIL_ROOT		4
#define	MSG_FOLDER_VOICEMAIL_ROOT	5
#define	MSG_FOLDER_IM_ROOT			6

#define MSG_FOLDER_INBOX	7
#define	MSG_FOLDER_OUTBOX	8
#define	MSG_FOLDER_SENT	9
#define	MSG_FOLDER_DRAFT	10
#define	MSG_FOLDER_TRASH	11
#define	MSG_FOLDER_UNSENT	12
#define	MSG_FOLDER_ARCHIVE	13
#define	MSG_FOLDER_SEARCH	14


#define MSG_FOLDER_REPORT  15
#define MSG_FOLDER_TEMPLATE 16

#ifndef __cplusplus
#ifndef _BOOL_
#define _BOOL_ 1
  typedef enum
  { false, true } bool;
#endif
#endif


  typedef uint32_t msg_folder_system_type_t;


  typedef struct _msg_folder_t msg_folder_t;



  typedef enum
  {
    MSG_FOLDER_SYSTEM,
    MSG_FOLDER_USER,
  } msg_folder_type_t;


  typedef struct
  {
    /** msg status*/
    msg_status_t msgstatus;

   /** msg with msgstatus status should be stored into 
     system folder with systype type*/
    msg_folder_system_type_t systype;
  } msg_disp_policy_t;

/********************msg dispatching policy discover*************
*/

/**
*@param number				returned number of pairs of msg status and system folder type.
*@param msg_disp_policy_t	returned array of pairs of msg status and system folder type.
*/
  msg_err_t msg_policy_discover (msg_sid_t sid, uint32_t *number,
                                 msg_disp_policy_t **policy);


/*************************msg folder manipulation ************
*/


  msg_folder_t *msg_folder_new (msg_sid_t sid, msg_fid_t parentid);
  msg_err_t msg_folder_release (msg_sid_t sid, msg_folder_t *folder);

/** a group of setting functions 
*/
  msg_err_t msg_folder_set_label (msg_sid_t sid, msg_folder_t *folder,
                                  char *label);

  msg_err_t msg_folder_set_maxbytes (msg_sid_t sid, msg_folder_t *folder,
                                     uint32_t maxbytes);

  msg_err_t msg_folder_set_msg_type (msg_sid_t sid, msg_folder_t *folder,
                                     msg_type_t type);


/** a group of getting functions
*/
  msg_fid_t msg_folder_get_id (msg_sid_t sid, msg_folder_t *folder);
  const char *msg_folder_get_label (msg_sid_t sid, msg_folder_t *folder);
  msg_fid_t msg_folder_get_parent (msg_sid_t sid, msg_folder_t *folder);
  msg_folder_type_t msg_folder_get_flag (msg_sid_t sid, msg_folder_t *folder);
  msg_type_t msg_folder_get_msg_type (msg_sid_t sid, msg_folder_t *folder);
  uint32_t msg_folder_get_maxbytes (msg_sid_t sid, msg_folder_t *folder);
  uint32_t msg_folder_get_has_children (msg_sid_t sid,
                                        const msg_folder_t *folder);

  uint32_t msg_folder_get_currentsize (msg_sid_t sid, msg_folder_t *folder);

/** add a folder as its child
  replace the parentid in msg_folder_t structure with the input parameter parentid
*/
  msg_err_t msg_folder_add_child (msg_sid_t sid, msg_fid_t parentid,
                                  msg_folder_t *folder);

/** remove a child folder
*/

  msg_err_t msg_folder_remove_child (msg_sid_t sid, msg_fid_t parentid,
                                     msg_fid_t childid);

/**list all children
* @param number			number of returned folders
* @param folderarray	returned folder array
*/
  msg_err_t msg_folder_list_children (msg_sid_t sid, msg_fid_t folder_id,
                                      uint32_t *number,
                                      msg_folder_t ***folderarray);

/** add a new message into a specified folder
*/

/** in this function, will not check dispatch policy, and will not change message status
  */
  msg_err_t msg_folder_add_msg (msg_sid_t sid, msg_fid_t fid,
                                msg_message_t *message);

/**remove a message from a folder. the message record in database is not deleted
*/
  msg_err_t msg_folder_remove_msg (msg_sid_t sid, msg_fid_t fid,
                                   msg_msgid_t id);

/** move a message from one folder to another
*/

  msg_err_t msg_folder_move_msg (msg_sid_t sid, msg_fid_t from, msg_fid_t to,
                                 msg_msgid_t id);
/**count the number of messages having a specified status in this folder
*/
  uint32_t msg_folder_count_msgs (msg_sid_t sid, msg_fid_t fid,
                                  msg_status_t status, bool recursive);

/** return headers and attributes of all messages in this folder.
* @param number		number of returned msgs
* @param msgarray	returned msg array, each msg will not return body.
*/
  msg_err_t msg_folder_list_msg_dig (msg_sid_t sid, msg_fid_t fid,
                                     uint32_t *number,
                                     msg_message_t ***msgarray);


/** search all messages in the current folder which match a specific querying criteria.
*/
  msg_err_t msg_folder_search_msgs (msg_sid_t sid, msg_fid_t fid,
                                    msg_message_t *match, uint32_t *number,
                                    msg_message_t ***msgarray);



#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __LIPS_MSG_FOLDER_H */

