/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "conffile.h"

#define CONF_PATH_VAR  "SMS_CONF_PATH"
#define TMP_DB_FILE    "/tmp/tmpmessage.db"

gint
_read_msg_conffile (msg_conf_t *conf)
{
  FILE *fp = NULL;
  gchar buf[MAX_BUF_LEN];
  char *prefix = getenv (CONF_PATH_VAR)? getenv(CONF_PATH_VAR):PREFIX;
    


  memset (buf, 0x00, sizeof (buf));

  if ((!conf) || (!conf->conf_file))
    return ERR_INVALID_ARG;

  fp = fopen (conf->conf_file, "r");
  if (!fp)
    return ERR_OPEN_FILE;

  while (fgets (buf, MAX_BUF_LEN, fp))
  {

    if (*(buf + strlen (buf) - 1) == '\n')
      *(buf + strlen (buf) - 1) = 0;

    if (strncmp (buf, "dbfile=", strlen ("dbfile=")) == 0)
      conf->db_file =
        g_strdup_printf ("%s/%s", prefix, buf + strlen ("dbfile="));

    else if (strncmp (buf, "msgtmpl=", strlen ("msgtmpl=")) == 0)
      conf->msg_tmpl =
        g_strdup_printf ("%s/%s", prefix, buf + strlen ("msgtmpl="));

    else if (strncmp (buf, "foldertmpl=", strlen ("foldertmpl=")) == 0)
      conf->folder_tmpl =
        g_strdup_printf ("%s/%s", prefix, buf + strlen ("foldertmpl="));

    else if (strncmp (buf, "policytmpl=", strlen ("policytmpl=")) == 0)
      conf->policy_tmpl =
        g_strdup_printf ("%s/%s", prefix, buf + strlen ("policytmpl="));

    else if (strncmp (buf, "tmpdb=", strlen ("tmpdb=")) == 0)
      conf->tmp_db = g_strdup_printf ("%s/%s", prefix, buf + strlen ("tmpdb="));
  }

  fclose (fp);

  return 0;

}



msg_conf_t *
_msg_conf_new ()
{
  char *prefix = getenv (CONF_PATH_VAR)? getenv(CONF_PATH_VAR):PREFIX;

  msg_conf_t *conf = NULL;

  conf = g_malloc0 (sizeof (msg_conf_t));
  if (!conf)
    return NULL;

  conf->conf_file = g_strdup_printf ("%s/%s", prefix, MSG_CONF_FILE);
  return conf;

}

gint
_msg_conf_release (msg_conf_t *conf)
{
  g_free (conf->conf_file);
  g_free (conf->db_file);
  g_free (conf->msg_tmpl);
  g_free (conf->folder_tmpl);
  g_free (conf->policy_tmpl);
  g_free (conf->tmp_db);

  g_free (conf);

  return 0;
}

gchar *
_get_tmpdb_location_alloc ()
{
/*
	msg_conf_t* conf = _msg_conf_new(MSG_CONF_FILE);
	if(!conf)
		return NULL;

	_read_msg_conffile(conf);
	gchar* ret = g_strdup(conf->tmp_db);
	_msg_conf_release(conf);

	return ret;	
*/
  return g_strdup (TMP_DB_FILE);
}

gchar *
_get_dbfile_location_alloc ()
{
  msg_conf_t *conf = _msg_conf_new (MSG_CONF_FILE);

  if (!conf)
    return NULL;

  _read_msg_conffile (conf);
  gchar *ret = g_strdup (conf->db_file);

  _msg_conf_release (conf);

  return ret;

}

gchar *
_get_msg_tmpl_location_alloc ()
{

  msg_conf_t *conf = _msg_conf_new (MSG_CONF_FILE);

  if (!conf)
    return NULL;

  _read_msg_conffile (conf);
  gchar *ret = g_strdup (conf->msg_tmpl);

  _msg_conf_release (conf);

  return ret;
}

gchar *
_get_folder_tmpl_location_alloc ()
{

  msg_conf_t *conf = _msg_conf_new (MSG_CONF_FILE);

  if (!conf)
    return NULL;

  _read_msg_conffile (conf);
  gchar *ret = g_strdup (conf->folder_tmpl);

  _msg_conf_release (conf);

  return ret;
}

gchar *
_get_policy_tmpl_location_alloc ()
{

  msg_conf_t *conf = _msg_conf_new (MSG_CONF_FILE);

  if (!conf)
    return NULL;

  _read_msg_conffile (conf);
  gchar *ret = g_strdup (conf->policy_tmpl);

  _msg_conf_release (conf);

  return ret;
}

