/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <string.h>

#include "msgevent.h"
#include "msgfolder.h"
#include "trans-sms-opt.h"
#include "print-err.h"
#include "id-generate.h"
#include "folder-opt.h"



msg_evt_t *event_generate (msg_sid_t sid, msg_evt_type_t evt_type,
                           trans_sms_t *trans_data);

int
dbus_event_process (uint32_t evt_arg1, uint32_t evt_arg2)
{

  msg_type_t evt_type = evt_arg1;
  trans_sms_t *trans_data = (trans_sms_t *) evt_arg2;
  msg_sid_t sid = 0;

  msg_evt_t *evt = NULL;

  if (evt_arg2 == 0)
  {
    print_err ("no event data is delivered");
    return -1;
  }

#ifdef MSGDEBUG
  g_print ("%s:trans_data is %d\n", __FUNCTION__, (uint32_t) trans_data);
#endif

  evt = event_generate (sid, evt_type, trans_data);
  if (!evt)
  {
    print_err ("error in contructing event");
    return -1;
  }


  msg_evt_call (sid, evt);

  msg_evt_release (sid, evt);

  return 0;


}

msg_evt_t *
event_generate (msg_sid_t sid, msg_evt_type_t evt_type,
                trans_sms_t *trans_data)
{
  msg_message_t *message = NULL;
  void *evt_data = NULL;
  msg_type_t msg_type = MSG_SMS;
  msg_evt_t *evt = NULL;
  uint32_t ref_no = 0;

  switch (evt_type)
  {

    case MSG_EVT_MSG_INCOMING:
      message = _new_message_from_trans_data (trans_data);

      msg_folder_add_msg (sid, MSG_FOLDER_INBOX, message);
      evt_data = g_malloc (sizeof (uint32_t));
      memcpy (evt_data, &message->id, sizeof (uint32_t));
      msg_release (sid, message);
      break;

    case MSG_EVT_MSG_SENT_SUCCESS:
      evt_data = g_malloc (sizeof (uint32_t));
      ref_no = trans_data->ref_no;
      if (_update_msg_refno (trans_data->serial_no, ref_no) == -1)
      {

        print_err ("SENT_SUCCESS, update database failed");
        return NULL;
      }
#ifdef MSGDEBUG
      g_print ("%s: SENT_SUCCESS, ref no update\n", __FUNCTION__);
#endif

      memcpy (evt_data, &trans_data->serial_no, sizeof (uint32_t));
      break;

    case MSG_EVT_MSG_SENT_FAILED:
      evt_data = g_malloc (sizeof (uint32_t));
      memcpy (evt_data, &trans_data->serial_no, sizeof (uint32_t));
      break;

    case MSG_EVT_MSG_REACH_CONFIRMED:
      evt_data = g_malloc (sizeof (uint32_t));
      ref_no = trans_data->ref_no;
      *(uint32_t *) evt_data = _msgid_from_refno (ref_no);

      /** search all records in database to find the propriate 
				*record, -1 will be returned if any error occurs, and 0 for no record found*/

#ifdef MSGDEBUG
      g_print ("%s: REACH_CONFIRMED, get the msgid %d here\n", __FUNCTION__,
               *(uint32_t *) evt_data);
#endif

      break;

    case MSG_EVT_MSG_REACH_FAILED:
      evt_data = (uint8_t *) g_malloc (sizeof (uint8_t));
      memcpy (evt_data, &trans_data->ref_no, sizeof (uint8_t));
      break;

    default:
      print_err ("event type cannot be recognized");
      return NULL;
  }

  if (_msg_evt_get (sid, &evt, evt_type, msg_type, evt_data) != 0)
    return NULL;

  return evt;
}

