/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <libintl.h>
#include <glib.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-bindings.h>
#define _(x) gettext(x)


typedef struct
{
  gchar *interface;
  gchar *method;
  void (*callback) ();
} t_listener;

static GList *listeners = NULL;
static DBusConnection *bus = NULL;


static DBusHandlerResult
msg_filter (DBusConnection *connection, DBusMessage *message, void *user_data)
{
  GMainLoop *mainloop = (GMainLoop *) user_data;
  GList *iter;


  for (iter = listeners; iter; iter = iter->next)
  {
    t_listener *li = (t_listener *) iter->data;

    if (dbus_message_is_signal (message, li->interface, li->method))
    {
      li->callback (message);
      return DBUS_HANDLER_RESULT_HANDLED;
    }
    else if (dbus_message_is_method_call (message, li->interface, li->method))
    {
      li->callback (message);
      return DBUS_HANDLER_RESULT_HANDLED;
    }
  }

  /* A signal from the bus saying we are about to be disconnected */
  if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, "Disconnected"))
  {
    g_main_loop_quit (mainloop);
    return DBUS_HANDLER_RESULT_HANDLED;
  }

  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
}


void
dbus_add_msg_listener_mc (const gchar *interface, const gchar *method,
                          void (*callback) ())
{

        /** fix me, using hash table may be better. */

  t_listener *new_listener = g_malloc0 (sizeof (t_listener));
  gchar *filterstr =
    g_strdup_printf ("type='method_call',interface='%s'", interface);

  new_listener->interface = g_strdup (interface);
  new_listener->method = g_strdup (method);
  new_listener->callback = callback;

  listeners = g_list_append (listeners, (gpointer) new_listener);

  dbus_bus_add_match (bus, filterstr, NULL);
  g_free (filterstr);

}

void
dbus_add_msg_listener (const gchar *interface, const gchar *method,
                       void (*callback) ())
{

        /** fix me, using hash table may be better. */

  t_listener *new_listener = g_malloc0 (sizeof (t_listener));
  gchar *filterstr =
    g_strdup_printf ("type='signal',interface='%s'", interface);

  new_listener->interface = g_strdup (interface);
  new_listener->method = g_strdup (method);
  new_listener->callback = callback;

  listeners = g_list_append (listeners, (gpointer) new_listener);

  dbus_bus_add_match (bus, filterstr, NULL);
  g_free (filterstr);

}

DBusConnection *
dbus_init (void)
{
  DBusError error;
  GMainLoop *mainloop;

  mainloop = g_main_loop_new (NULL, TRUE);
  dbus_error_init (&error);

  bus = dbus_bus_get (DBUS_BUS_SESSION, &error);
  if (!bus)
  {
    g_warning ("Failed to connect to the D-BUS daemon: %s", error.message);
    dbus_error_free (&error);
    return NULL;
  }

  dbus_connection_setup_with_g_main (bus, NULL);

  /* we do not request a namespace to support multiple listeners to these signals */

  dbus_connection_add_filter (bus, msg_filter, mainloop, NULL);

  g_main_loop_unref (mainloop);
  return bus;
}

