/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <database.h>

#include "message-opt.h"
#include "folder-opt.h"
#include "conffile.h"
#include "policy-tmpl.h"
#include "folder-tmpl.h"
#include "print-err.h"


int
_test_policy_add (msg_status_t msg_status, msg_folder_system_type_t folder_type)
{

  RecordDB *database = NULL;
  Record *record = NULL;
  int ret = 0;

  gchar *policy_tmpl = _get_policy_tmpl_location_alloc ();

  if (!policy_tmpl)
    return MSG_ERROR_READ_CONF_FILE;


        /**get database file location*/
  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    g_free (policy_tmpl);
    return MSG_ERROR_READ_CONF_FILE;
  }

        /** open database*/
  FieldTemplate *template = field_template_new ();

  if (field_template_parse_file (template, policy_tmpl) != TRUE)
  {
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_POLICY, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }

  record = record_new (template);

  record_set_single_field (record, POLICY_MESSAGE_STATUS, &msg_status);
  record_set_single_field (record, POLICY_FOLDER_TYPE, &folder_type);

  record_db_add_record (database, record);

end:
  g_free (policy_tmpl);
  g_free (db_file);

  field_template_free (template);

  record_free (record);


  if (database)
    record_db_close (database);

  record_db_free (database);
  return ret;

}


msg_err_t
_policy_discover (msg_sid_t sid, uint32_t *number, msg_disp_policy_t **policy)
{
  RecordDB *database = NULL;
  Record *current = NULL;
  Iterator *iter = NULL;
  msg_err_t ret = 0;
  gint outputs[] = { POLICY_NUM, POLICY_MESSAGE_STATUS, POLICY_FOLDER_TYPE };
  *number = 0;
  msg_disp_policy_t *policy_a = NULL;
  gint i = 0;

  gchar *policy_tmpl = _get_policy_tmpl_location_alloc ();

  if (!policy_tmpl)
  {
    print_err ("getting policy template file location");
    return MSG_ERROR_READ_CONF_FILE;
  }

        /**get database file location*/
  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    print_err ("getting database file location");
    g_free (policy_tmpl);
    return MSG_ERROR_READ_CONF_FILE;
  }

        /** open database*/
  FieldTemplate *template = field_template_new ();

  if (field_template_parse_file (template, policy_tmpl) != TRUE)
  {
    print_err ("parsing template file");
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_POLICY, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    print_err ("database open");
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }


        /** query record as keyword id from database*/
  iter = iterator_new_full ((ElementFreeFunc) record_free);



  record_db_get_records (database, iter, NULL,
                         sizeof (outputs) / sizeof (gint), outputs);

  *number = iterator_size (iter);
  if (*number == 0)
    goto end;

  policy_a = g_new0 (msg_disp_policy_t, *number);
  if (!policy_a)
  {
    print_err ("cannot allcate memory");
    ret = MSG_ERROR_MEM_ALLOC;
    goto end;
  }

  for (iterator_to_first (iter); !iterator_at_last (iter); iterator_next (iter))
  {
    current = (Record *) iterator_current (iter);
    policy_a[i].msgstatus =
      *(uint32_t *) record_get_single_field_static (current,
                                                    POLICY_MESSAGE_STATUS);
    policy_a[i].systype =
      *(uint32_t *) record_get_single_field_static (current,
                                                    POLICY_FOLDER_TYPE);
    i++;
  }

  *policy = policy_a;


end:
  g_free (policy_tmpl);
  g_free (db_file);

  field_template_free (template);

  if (iter)
    iterator_free (iter);

  if (database)
    record_db_close (database);

  record_db_free (database);
  return ret;

}

int
_add_folder (msg_fid_t pid, msg_folder_t *folder)
{

  gint ret = 0;

  FieldTemplate *template = NULL;

  RecordDB *database = NULL;
  Record *record = NULL;
  Record *parent_record = NULL;
  gint updates[] = { FOLDER_CHLDNUM };
  uint32_t child_num = 0;
  msg_fid_t fid = folder->id;

  gchar *folder_tmpl = _get_folder_tmpl_location_alloc ();

  if (!folder_tmpl)
    return -1;

  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    g_free (folder_tmpl);
    return -1;
  }

  template = field_template_new ();

  if (field_template_parse_file (template, folder_tmpl) != TRUE)
  {
    ret = -1;
    goto end;
  }

  record = record_new (template);
  if (!record)
  {
    ret = -1;
    goto end;
  }

  _record_from_folder (folder, record);

  record_print (record);

  database = record_db_new (db_file, PREFIX_FOLDER, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = -1;
    goto end;
  }

  if (record_db_add_record (database, record) != 0)
  {
    ret = -1;
    goto end;
  }

  if (folder->id == 0)
    goto end;

  parent_record = record_new (template);
  if (!parent_record)
  {
    ret = -1;
    goto end;
  }

  if (record_db_get_record (database, parent_record, &pid) != 0)
  {
    record_db_remove_record (database, &fid);
    ret = -1;
    goto end;
  }


  child_num =
    *(uint32_t *) record_get_single_field_static (parent_record,
                                                  FOLDER_CHLDNUM) + 1;

  record_set_single_field (parent_record, FOLDER_CHLDNUM, &child_num);


  if (record_db_update_record
      (database, parent_record, sizeof (updates) / sizeof (gint), updates) != 0)
  {
    record_db_remove_record (database, &fid);
    ret = -1;
    goto end;
  }



end:
  g_free (db_file);
  g_free (folder_tmpl);
  record_free (record);
  record_free (parent_record);
  record_db_close (database);
  record_db_free (database);
  field_template_free (template);

  return ret;
}

int
_remove_folder (msg_fid_t pid, msg_fid_t fid)
{

  gint ret = 0;

  FieldTemplate *template = NULL;

  RecordDB *database = NULL;
  Record *record = NULL;
  Record *parent_record = NULL;
  gint updates[] = { FOLDER_CHLDNUM };
  uint32_t child_num = 0;

  gchar *folder_tmpl = _get_folder_tmpl_location_alloc ();

  if (!folder_tmpl)
    return -1;

  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    g_free (folder_tmpl);
    return -1;
  }

  template = field_template_new ();

  if (field_template_parse_file (template, folder_tmpl) != TRUE)
  {
    ret = -1;
    goto end;
  }

  record = record_new (template);
  if (!record)
  {
    ret = -1;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_FOLDER, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = -1;
    goto end;
  }

  if (record_db_get_record (database, record, &fid) != 0)
  {
    ret = -1;
    goto end;
  }

  if (*(uint32_t *) record_get_single_field_static (record, FOLDER_PID) != pid)
  {
    ret = -3;
    goto end;
  }


  parent_record = record_new (template);
  if (!parent_record)
  {
    ret = -1;
    goto end;
  }

  if (record_db_get_record (database, parent_record, &pid) != 0)
  {
    ret = -1;
    goto end;
  }

  child_num =
    *(uint32_t *) record_get_single_field_static (parent_record,
                                                  FOLDER_CHLDNUM) - 1;

  record_set_single_field (parent_record, FOLDER_CHLDNUM, &child_num);

  if (record_db_update_record
      (database, parent_record, sizeof (updates) / sizeof (gint), updates) != 0)
  {
    ret = -1;
    goto end;
  }

  if (record_db_remove_record (database, &fid) != 0)
  {
    ret = -2;
    goto end;
  }

end:
  g_free (db_file);
  g_free (folder_tmpl);
  record_free (record);
  record_free (parent_record);
  record_db_close (database);
  record_db_free (database);
  field_template_free (template);

  return ret;
}

int
_folder_add_size (msg_fid_t fid, int num)
{

  gint ret = 0;

  FieldTemplate *template = NULL;

  RecordDB *database = NULL;
  Record *record = NULL;
  gint updates[] = { FOLDER_CURRTSIZE };
  uint32_t size = 0;
  uint32_t max_size = 0;

  gchar *folder_tmpl = _get_folder_tmpl_location_alloc ();

  if (!folder_tmpl)
    return -1;

  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    g_free (folder_tmpl);
    return -1;
  }

  template = field_template_new ();

  if (field_template_parse_file (template, folder_tmpl) != TRUE)
  {
    ret = -1;
    goto end;
  }

  record = record_new (template);
  if (!record)
  {
    ret = -1;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_FOLDER, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = -1;
    goto end;
  }


  record = record_new (template);
  if (!record)
  {
    ret = -1;
    goto end;
  }

  if (record_db_get_record (database, record, &fid) != 0)
  {
    ret = -1;
    goto end;
  }

  size =
    *(uint32_t *) record_get_single_field_static (record,
                                                  FOLDER_CURRTSIZE) + num;

  max_size =
    *(uint32_t *) record_get_single_field_static (record, FOLDER_MAXBYTES);
  if ((size < 0) || (size > max_size))
  {
    ret = -2;
    goto end;
  }

  record_set_single_field (record, FOLDER_CURRTSIZE, &size);

  if (record_db_update_record
      (database, record, sizeof (updates) / sizeof (gint), updates) != 0)
  {
    ret = -1;
    goto end;
  }


end:
  g_free (db_file);
  g_free (folder_tmpl);
  record_free (record);
  record_db_close (database);
  record_db_free (database);
  field_template_free (template);

  return ret;
}

int
_add_childnum (msg_fid_t pid, int num)
{

  gint ret = 0;

  FieldTemplate *template = NULL;

  RecordDB *database = NULL;
  Record *record = NULL;
  gint updates[] = { FOLDER_CHLDNUM };
  uint32_t child_num = 0;

  gchar *folder_tmpl = _get_folder_tmpl_location_alloc ();

  if (!folder_tmpl)
    return -1;

  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    g_free (folder_tmpl);
    return -1;
  }

  template = field_template_new ();

  if (field_template_parse_file (template, folder_tmpl) != TRUE)
  {
    ret = -1;
    goto end;
  }

  record = record_new (template);
  if (!record)
  {
    ret = -1;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_FOLDER, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = -1;
    goto end;
  }


  record = record_new (template);
  if (!record)
  {
    ret = -1;
    goto end;
  }

  if (record_db_get_record (database, record, &pid) != 0)
  {
    ret = -1;
    goto end;
  }

  child_num =
    *(uint32_t *) record_get_single_field_static (record, FOLDER_CHLDNUM) + num;

  record_set_single_field (record, FOLDER_CHLDNUM, &child_num);

  if (record_db_update_record
      (database, record, sizeof (updates) / sizeof (gint), updates) != 0)
  {
    ret = -1;
    goto end;
  }


end:
  g_free (db_file);
  g_free (folder_tmpl);
  record_free (record);
  record_db_close (database);
  record_db_free (database);
  field_template_free (template);

  return ret;
}


int
_is_folder_empty (msg_fid_t fid)
{

  uint32_t ret = 1;
  uint32_t num = 0;

  FieldTemplate *template = NULL;

  RecordDB *database = NULL;
  Record *record = NULL;

  gchar *folder_tmpl = _get_folder_tmpl_location_alloc ();

  if (!folder_tmpl)
    return -1;

  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    g_free (folder_tmpl);
    return -1;
  }

  template = field_template_new ();

  if (field_template_parse_file (template, folder_tmpl) != TRUE)
  {
    ret = -1;
    goto end;
  }

  record = record_new (template);
  if (!record)
  {
    ret = -1;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_FOLDER, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = -1;
    goto end;
  }


  if (record_db_get_record (database, record, &fid) != 0)
  {
    ret = -1;
    goto end;
  }

  num = *(uint32_t *) record_get_single_field_static (record, FOLDER_CHLDNUM);
  if (num != 0)
  {
    ret = 0;
    goto end;
  }

  num = *(uint32_t *) record_get_single_field_static (record, FOLDER_CURRTSIZE);
  if (num != 0)
    ret = 0;

end:
  g_free (db_file);
  g_free (folder_tmpl);
  record_free (record);
  record_db_close (database);
  record_db_free (database);
  field_template_free (template);

  return ret;
}

int
_record_from_folder (msg_folder_t *folder, Record *record)
{
  uint32_t type = folder->type;
  uint32_t flag = folder->folder_flag;

  if (folder->id != -1)
    record_set_single_field (record, FOLDER_ID, &folder->id);

  record_set_single_field (record, FOLDER_LABEL, folder->label);
  record_set_single_field (record, FOLDER_PID, &folder->parent_id);
  record_set_single_field (record, FOLDER_CHLDNUM, &folder->children_num);
  record_set_single_field (record, FOLDER_FLDFLAG, &flag);
  record_set_single_field (record, FOLDER_TYPE, &type);
  record_set_single_field (record, FOLDER_CURRTSIZE, &folder->currentsize);
  record_set_single_field (record, FOLDER_MAXBYTES, &folder->maxbytes);

  return 0;
}


msg_err_t
_list_children_folder (msg_fid_t folder_id, uint32_t *number,
                       msg_folder_t ***folderarray)
{
  RecordDB *database = NULL;
  Record *match = NULL;
  Iterator *iter = NULL;
  msg_err_t ret = 0;
  gint outputs[] =
    { FOLDER_ID, FOLDER_LABEL, FOLDER_PID, FOLDER_FLDFLAG, FOLDER_CHLDNUM,
    FOLDER_TYPE, FOLDER_CURRTSIZE, FOLDER_MAXBYTES
  };
  *number = 0;
  gint i = 0;

  gchar *folder_tmpl = _get_folder_tmpl_location_alloc ();

  if (!folder_tmpl)
  {
    print_err ("reading configure file");
    return MSG_ERROR_READ_CONF_FILE;
  }


        /**get database file location*/
  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    g_free (folder_tmpl);
    print_err ("reading configure file");
    return MSG_ERROR_READ_CONF_FILE;
  }

        /** open database*/
  FieldTemplate *template = field_template_new ();

  if (field_template_parse_file (template, folder_tmpl) != TRUE)
  {
    print_err ("database manipulate");
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_FOLDER, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    print_err ("database manipulate");
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }


        /** query record as keyword id from database*/
  iter = iterator_new_full ((ElementFreeFunc) record_free);

  match = record_new (template);
  if (!match)
  {
    print_err ("cannot allocate memory");
    ret = MSG_ERROR_MEM_ALLOC;
    goto end;
  }

  record_set_single_field (match, FOLDER_PID, &folder_id);


  ret =
    record_db_get_records (database, iter, match,
                           sizeof (outputs) / sizeof (gint), outputs);
  if (ret != DB_ERROR_NONE)
  {
    print_err ("database manipulate");
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }
  *number = iterator_size (iter);

  if (*number == 0)
    goto end;


  *folderarray = g_new0 (msg_folder_t *, *number);
  if (!*folderarray)
  {
    print_err ("cannot allocate memory");
    ret = MSG_ERROR_MEM_ALLOC;
    goto end;
  }


  for (iterator_to_first (iter); !iterator_at_last (iter); iterator_next (iter))
  {
    (*folderarray)[i] = (msg_folder_t *) g_malloc0 (sizeof (msg_folder_t));
    Record *temp = (Record *) iterator_current (iter);

#ifdef _DEBUG
    record_print (temp);
#endif
    _record_to_folder (temp, (*folderarray)[i++]);
  }


end:
  g_free (folder_tmpl);
  g_free (db_file);

  field_template_free (template);

  record_free (match);

  if (iter)
    iterator_free (iter);

  if (database)
    record_db_close (database);

  record_db_free (database);
  return ret;
}

int
_record_to_folder (Record *record, msg_folder_t *folder)
{
  folder->id =
    *(msg_fid_t *) record_get_single_field_static (record, FOLDER_ID);
  folder->label =
    g_strdup (record_get_single_field_static (record, FOLDER_LABEL));
  folder->parent_id =
    *(msg_fid_t *) record_get_single_field_static (record, FOLDER_PID);
  folder->children_num =
    *(uint32_t *) record_get_single_field_static (record, FOLDER_CHLDNUM);
  folder->folder_flag =
    *(uint32_t *) record_get_single_field_static (record, FOLDER_FLDFLAG);
  folder->type =
    *(uint32_t *) record_get_single_field_static (record, FOLDER_TYPE);
  folder->currentsize =
    *(uint32_t *) record_get_single_field_static (record, FOLDER_CURRTSIZE);
  folder->maxbytes =
    *(uint32_t *) record_get_single_field_static (record, FOLDER_MAXBYTES);

  return 0;
}


msg_err_t
_change_message_parent (msg_fid_t from, msg_fid_t to, msg_msgid_t msgid)
{

  RecordDB *database = NULL;
  msg_err_t ret = 0;
  Record *record = NULL;
  gint update[] = { MESSAGE_PARENT };


  gchar *msg_tmpl = _get_msg_tmpl_location_alloc ();

  if (!msg_tmpl)
  {
    print_err ("reading configure file");
    return MSG_ERROR_READ_CONF_FILE;
  }


        /**get database file location*/
  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    g_free (msg_tmpl);
    print_err ("reading configure file");
    return MSG_ERROR_READ_CONF_FILE;
  }

        /** open database*/
  FieldTemplate *template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    print_err ("database manipulate");
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    print_err ("database manipulate");
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }

  record = record_new (template);
  if (record_db_get_record (database, record, &msgid) != DB_ERROR_NONE)
  {
    print_err ("no such record");
    ret = MSG_ERROR_NO_SUCH_RECORD;
    goto end;
  }

#ifdef MSGDEBUG
  printf ("%s: parent is %d\n", __FUNCTION__,
          *(msg_fid_t *) record_get_single_field_static (record,
                                                         MESSAGE_PARENT));
#endif

  if (*(msg_fid_t *) record_get_single_field_static (record, MESSAGE_PARENT)
      != from)
  {
    print_err ("wrong parent");
    ret = MSG_ERROR_WRONG_PARENT;            /**wrong parent*/
    goto end;
  }



  record_set_single_field (record, MESSAGE_PARENT, &to);
  record_db_update_record (database, record, sizeof (update) / sizeof (gint),
                           update);



end:
  g_free (msg_tmpl);
  g_free (db_file);

  field_template_free (template);
  record_free (record);

  if (database)
    record_db_close (database);


  record_db_free (database);
  return ret;
}


uint32_t
_count_msg_in_folder (msg_fid_t fid, msg_status_t status, char *db_file)
{
  RecordDB *database = NULL;
  Iterator *iter = NULL;
  gint outputs[] = { MESSAGE_PARENT, MESSAGE_ID };
  Record *match = NULL;
  uint32_t ret = 0;
  gchar *msg_tmpl = NULL;
  FieldTemplate *template = NULL;
  uint32_t stat = status;


  msg_tmpl = _get_msg_tmpl_location_alloc ();
  if (!msg_tmpl)
  {
    print_err ("reading configure file");
    return -1;
  }



        /** open database*/
  template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    print_err ("database manipulate");
    ret = -1;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    print_err ("database manipulate");
    ret = -1;
    goto end;
  }


  iter = iterator_new_full ((ElementFreeFunc) record_free);
  match = record_new (template);

  record_set_single_field (match, MESSAGE_PARENT, &fid);

  if (status)
    record_set_single_field (match, MESSAGE_STATUS, &stat);


  record_db_get_records (database, iter, match,
                         sizeof (outputs) / sizeof (gint), outputs);

  ret = iterator_size (iter);

end:
  g_free (msg_tmpl);

  field_template_free (template);
  iterator_free (iter);
  record_free (match);

  if (database)
    record_db_close (database);

  record_db_free (database);
  return ret;
}

uint32_t
_count_msg_in_folder_r (msg_fid_t fid, msg_status_t status)
{
  msg_folder_t **folder_a = NULL;
  uint32_t num = 0;
  uint32_t msg_sum = 0;
  gint i = 0;
  gint ret = 0;

  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    return -1;
  }


  if (_list_children_folder (fid, &num, &folder_a) != 0)
  {
    print_err ("listing child folder");
    return -1;
    goto end;
  }

  for (i = 0; i < num; i++)
  {
    ret = _count_msg_in_folder_r (folder_a[i]->id, status);
    if (ret == -1)
    {
      msg_sum = -1;
      goto end;
    }
    else
      msg_sum += ret;
  }

  ret = _count_msg_in_folder (fid, status, db_file);
  if (ret == -1)
  {
    msg_sum = -1;
    goto end;
  }
  else
    msg_sum += ret;


end:
  for (i = 0; i < num; i++)
    msg_folder_release (0, folder_a[i]);

  g_free (folder_a);
  g_free (db_file);

  return msg_sum;

}


msg_err_t
_list_message_dig (msg_fid_t fid, uint32_t *number,
                   msg_message_t ***msgarray, char *db_file,
                   msg_mem_t mem_type)
{

  RecordDB *database = NULL;
  Iterator *iter = NULL;
  gint outputs[] =
    { MESSAGE_SID, MESSAGE_ID, MESSAGE_STATUS, MESSAGE_TIME, MESSAGE_TYPE,
    MESSAGE_SIZE, MESSAGE_RECIPIENTS, MESSAGE_CC, MESSAGE_BCC, MESSAGE_ATTCH,
    MESSAGE_FROM, MESSAGE_SUBJECT
  };

  Record *match = NULL;
  msg_err_t ret = 0;
  gchar *msg_tmpl = NULL;
  FieldTemplate *template = NULL;
  gint i = 0;

  msg_tmpl = _get_msg_tmpl_location_alloc ();
  if (!msg_tmpl)
  {
    print_err ("reading configuration file");
    return MSG_ERROR_READ_CONF_FILE;
  }

        /** open database*/
  template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    print_err ("database manipulate");
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    print_err ("database manipulate");
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }


  iter = iterator_new_full ((ElementFreeFunc) record_free);
  match = record_new (template);

  record_set_single_field (match, MESSAGE_PARENT, &fid);

  record_db_get_records (database, iter, match,
                         sizeof (outputs) / sizeof (gint), outputs);

  *number = iterator_size (iter);
  if (*number == 0)
    goto end;

  *msgarray = g_new0 (msg_message_t *, *number);
  if (!*msgarray)
  {
    print_err ("cannot allocate memory");
    ret = MSG_ERROR_MEM_ALLOC;
    goto end;
  }

  for (iterator_to_first (iter); !iterator_at_last (iter); iterator_next (iter))
  {
    (*msgarray)[i] = (msg_message_t *) g_malloc0 (sizeof (msg_message_t));
    _message_dig_from_record ((Record *) iterator_current (iter),
                              (*msgarray)[i]);
    (*msgarray)[i]->mem_type = mem_type;
    i++;
  }


end:
  g_free (msg_tmpl);

  field_template_free (template);
  iterator_free (iter);
  record_free (match);

  if (database)
    record_db_close (database);

  record_db_free (database);
  return ret;
}

int
_message_dig_from_record (Record *record, msg_message_t *msg)
{
  if ((!record) || (!msg))
    return -1;

  msg->id =
    *(msg_msgid_t *) record_get_single_field_static (record, MESSAGE_ID);

  msg->sid =
    *(msg_msgsid_t *) record_get_single_field_static (record, MESSAGE_SID);

  msg->status =
    *(msg_status_t *) record_get_single_field_static (record, MESSAGE_STATUS);

  msg->timestamp =
    *(msg_time_t *) record_get_single_field_static (record, MESSAGE_TIME);

  msg->msg_type =
    *(msg_type_t *) record_get_single_field_static (record, MESSAGE_TYPE);

  msg->msg_size =
    *(msg_size_t *) record_get_single_field_static (record, MESSAGE_SIZE);

  msg->recipients =
    (char *) record_get_single_field (record, MESSAGE_RECIPIENTS);

  msg->cclist = (char *) record_get_single_field (record, MESSAGE_CC);

  msg->bcclist = (char *) record_get_single_field (record, MESSAGE_BCC);

  msg->attachlist = (char *) record_get_single_field (record, MESSAGE_ATTCH);

  msg->subject = (char *) record_get_single_field (record, MESSAGE_SUBJECT);

  msg->from = (char *) record_get_single_field (record, MESSAGE_FROM);

  return 0;


}


msg_err_t
_message_search (msg_fid_t fid, msg_message_t *match, int *number,
                 msg_message_t ***msgarray, char *db_file, msg_mem_t mem_type)
{

  RecordDB *database = NULL;
  Iterator *iter = NULL;
  gint outputs[] =
    { MESSAGE_SID, MESSAGE_ID, MESSAGE_STATUS, MESSAGE_TIME, MESSAGE_TYPE,
    MESSAGE_SIZE, MESSAGE_RECIPIENTS, MESSAGE_CC, MESSAGE_BCC, MESSAGE_ATTCH,
    MESSAGE_FROM, MESSAGE_SUBJECT, MESSAGE_BSIZE, MESSAGE_SIZE, MESSAGE_BODY
  };

  Record *match_record = NULL;
  msg_err_t ret = 0;
  gchar *msg_tmpl = NULL;
  FieldTemplate *template = NULL;
  gint i = 0;
  gint body_size = 0;
  Record *curr = NULL;
  msg_message_t **msg_a = NULL;


  msg_tmpl = _get_msg_tmpl_location_alloc ();
  if (!msg_tmpl)
  {
    print_err ("reading configure file");
    return MSG_ERROR_READ_CONF_FILE;
  }

        /** open database*/
  template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    print_err ("database manipulate");
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    print_err ("database manipulate");
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }


  iter = iterator_new_full ((ElementFreeFunc) record_free);
  match_record = record_new (template);

  _record_from_match_message (match, match_record);
  record_set_single_field (match_record, MESSAGE_PARENT, &fid);

  record_db_get_records (database, iter, match_record,
                         sizeof (outputs) / sizeof (gint), outputs);

  *number = iterator_size (iter);
  if (*number == 0)
    goto end;

  msg_a = g_new0 (msg_message_t *, *number);
  if (!msg_a)
  {
    print_err ("cannot allocate memory");
    ret = MSG_ERROR_MEM_ALLOC;
    goto end;
  }

  for (iterator_to_first (iter); !iterator_at_last (iter); iterator_next (iter))
  {
    curr = (Record *) iterator_current (iter);
    msg_a[i] = (msg_message_t *) g_malloc0 (sizeof (msg_message_t));
    _message_dig_from_record (curr, msg_a[i]);
    msg_a[i]->body = _get_body_from_record (curr, &body_size);
    msg_a[i]->body_size = body_size;
    msg_a[i]->mem_type = mem_type;
    i++;
  }

  *msgarray = msg_a;

end:
  g_free (msg_tmpl);

  field_template_free (template);
  iterator_free (iter);
  record_free (match_record);

  if (database)
    record_db_close (database);

  record_db_free (database);

  return ret;
}

uint32_t
_msgid_from_refno (uint32_t ref_no)
{
  uint32_t ret = 0;

  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
    return -1;

  ret = _message_search_by_refno (ref_no, db_file);
  g_free (db_file);

  if (ret != 0)
    return ret;
/*	
	db_file = _get_tmpdb_location_alloc();
	if(!db_file)
		return -1;

	ret = _message_search_by_refno(ref_no, db_file);
	g_free(db_file);
*/
  return ret;
}

uint32_t
_message_search_by_refno (uint32_t ref_no, char *db_file)
{

  RecordDB *database = NULL;
  Iterator *iter = NULL;
  gint outputs[] = { MESSAGE_ID };

  Record *match_record = NULL;
  uint32_t ret = 0;
  gchar *msg_tmpl = NULL;
  FieldTemplate *template = NULL;
  Record *curr = NULL;



  msg_tmpl = _get_msg_tmpl_location_alloc ();
  if (!msg_tmpl)
  {
    print_err ("reading configure file");
    return -1;

  }

        /** open database*/

  template = field_template_new ();
  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    print_err ("database manipulate");
    ret = -1;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    print_err ("database manipulate");
    ret = -1;
    goto end;
  }


  iter = iterator_new_full ((ElementFreeFunc) record_free);
  match_record = record_new (template);

  record_set_single_field (match_record, MESSAGE_REPORT, &ref_no);

  record_db_get_records (database, iter, match_record,
                         sizeof (outputs) / sizeof (gint), outputs);

  if (iterator_size (iter) == 0)
    goto end;


  curr = (Record *) iterator_current (iter);
  memcpy (&ret, record_get_single_field_static (curr, MESSAGE_ID),
          sizeof (uint32_t));

#ifdef MSGDEBUG
  g_print ("%s: record id is %d\n", __FUNCTION__, ret);
#endif


end:
  g_free (msg_tmpl);

  field_template_free (template);
  iterator_free (iter);
  record_free (match_record);

  if (database)
    record_db_close (database);

  record_db_free (database);

  return ret;
}

