/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdint.h>
#include <glib.h>
#include <stdio.h>
#include <string.h>

#include "message-opt.h"
#include "conffile.h"
#include "id-generate.h"
#include "print-err.h"
#include "trans-sms-opt.h"
#include "folder-opt.h"

extern int msg_service_msg_send (trans_sms_t *trans_data);

int
_init_message (msg_message_t *msg)
{

  msg->id = _id_generate (PREFIX_MESSAGE);
  msg->status = MSG_DRAFT;
  msg->sys_flag = 0;

  return 0;
}


msg_message_t *
_new_msg_by_id (msg_sid_t sid, msg_msgid_t id, char *db_file)
{

  RecordDB *database = NULL;
  Record *record_result = NULL;
  msg_message_t *msg = NULL;

        /**get message template file location*/
  gchar *msg_tmpl = _get_msg_tmpl_location_alloc ();

  if (!msg_tmpl)
  {
    print_err ("getting message template file location");
    return NULL;
  }


        /** open database*/
  FieldTemplate *template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    print_err ("parsing template file");
    goto end;
  }


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    print_err ("database open");
    goto end;
  }

        /** query record as keyword id from database*/

  record_result = record_new (template);

  record_db_get_record (database, record_result, &id);

  if (!record_get_single_field_static (record_result, MESSAGE_ID))
    goto end;

  msg = (msg_message_t *) g_malloc0 (sizeof (msg_message_t));
  if (!msg)
  {
    print_err ("Cannot alloc memory");
    goto end;
  }

  _message_dig_from_record (record_result, msg);

  msg->body = _get_body_from_record (record_result, &msg->body_size);

end:
  g_free (msg_tmpl);

  field_template_free (template);


  if (record_result)
    record_free (record_result);

  if (database)
    record_db_close (database);

  record_db_free (database);

  return msg;
}

void *
_get_body_by_id (msg_sid_t sid, msg_msgid_t id, int *body_size, gchar *db_file)
{

  RecordDB *database = NULL;
  Record *record_result = NULL;
  void *ret = NULL;

        /**get message template file location*/
  gchar *msg_tmpl = _get_msg_tmpl_location_alloc ();

  if (!msg_tmpl)
  {
    print_err ("getting message template file location");
    return NULL;
  }

        /** open database*/
  FieldTemplate *template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    print_err ("parsing template file");
    goto end;
  }


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    print_err ("database open");
    goto end;
  }

        /** query record as keyword id from database*/

  record_result = record_new (template);

  record_db_get_record (database, record_result, &id);
  if (!record_get_single_field_static (record_result, MESSAGE_ID))
    goto end;



  ret = _get_body_from_record (record_result, body_size);

end:
  g_free (msg_tmpl);

  field_template_free (template);


  if (record_result)
    record_free (record_result);

  if (database)
    record_db_close (database);

  record_db_free (database);
  return ret;
}

void *
_get_body_from_record (Record *record, int *body_size)
{
  void *ret = NULL;
  gint num = 0;
  gint i = 0;
  gint size = 0;

  *body_size =
    *(msg_size_t *) record_get_single_field_static (record, MESSAGE_BSIZE);
  num = record_multiple_field_size ((Record *) record, MESSAGE_BODY);

  if (*body_size == 0)
    goto end;

  size = *body_size;

        /**construct message body from record structure*/
  ret = g_malloc0 (*body_size);
  if (!ret)
    goto end;

  void *p = ret;

  for (i = 0; i < num - 1; i++)
  {
    memcpy (p, record_get_multiple_field_static (record, MESSAGE_BODY, i),
            BODY_UNIT_LENGTH);
    p += BODY_UNIT_LENGTH;
    size -= BODY_UNIT_LENGTH;
  }
  memcpy (p, record_get_multiple_field_static (record, MESSAGE_BODY, i), size);

end:
  return ret;

}

gint
_send_message (msg_message_t *message)
{
  trans_sms_t trans_data;
  int ret = 0;

  memset (&trans_data, 0x00, sizeof (trans_sms_t));


  _message_to_trans_data (message, &trans_data);

  if (msg_service_msg_send (&trans_data) != 0)
  {
    ret = -1;
  }

  g_free (trans_data.body);

#ifdef MSGDEBUG
  g_print ("%s:function returned\n", __FUNCTION__);
#endif

  return ret;
}


gint
_store_message (msg_message_t * message, msg_fid_t fid)
{
  gint ret = 0;

  FieldTemplate *template = NULL;

  RecordDB *database = NULL;
  Record *record = NULL;

  gchar *msg_tmpl = _get_msg_tmpl_location_alloc ();

  if (!msg_tmpl)
    return -1;

  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    g_free (msg_tmpl);
    return -1;
  }

  template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    ret = -1;
    goto end;
  }

  record = record_new (template);
  if (!record)
  {
    ret = -1;
    goto end;
  }

  _record_from_message (message, record);

  record_set_single_field (record, MESSAGE_PARENT, &fid);


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = -1;
    goto end;
  }

  if (record_db_add_record (database, record) != 0)
    ret = -1;

end:
  g_free (db_file);
  g_free (msg_tmpl);
  record_free (record);
  record_db_close (database);
  record_db_free (database);
  field_template_free (template);

  return ret;
}

gint
_delete_message (msg_msgid_t msgid, msg_fid_t fid)
{
  gint ret = 0;

  FieldTemplate *template = NULL;
  Record *record = NULL;
  RecordDB *database = NULL;

  gchar *msg_tmpl = _get_msg_tmpl_location_alloc ();

  if (!msg_tmpl)
    return -1;

  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    g_free (msg_tmpl);
    return -1;
  }

  template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    ret = -1;
    goto end;
  }

  record = record_new (template);
  if (!record)
  {
    ret = -1;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = -1;
    goto end;
  }

  record_db_get_record (database, record, &msgid);

  if (!record_get_single_field_static (record, MESSAGE_ID))
  {
    ret = -3;
    goto end;
  }

  if (*(msg_fid_t *) record_get_single_field_static (record, MESSAGE_PARENT)
      != fid)
  {
    ret = -2;
    goto end;
  }


  if (record_db_remove_record (database, &msgid) != 0)
    ret = -1;

end:
  g_free (db_file);
  g_free (msg_tmpl);
  record_db_close (database);
  record_db_free (database);
  field_template_free (template);

  return ret;
}

gint
_record_from_message (msg_message_t *message, Record *record)
{
  uint32_t type = message->msg_type;
  uint32_t status = message->status;

  guchar body_last[BODY_UNIT_LENGTH];

  memset (body_last, 0x00, BODY_UNIT_LENGTH);

  record_set_single_field (record, MESSAGE_ID, &message->id);
  record_set_single_field (record, MESSAGE_SID, &message->sid);
  record_set_single_field (record, MESSAGE_STATUS, &status);
  record_set_single_field (record, MESSAGE_TIME, &message->timestamp);
  record_set_single_field (record, MESSAGE_TYPE, &type);
  record_set_single_field (record, MESSAGE_SIZE, &message->msg_size);


  record_set_single_field (record, MESSAGE_RECIPIENTS, message->recipients);
  record_set_single_field (record, MESSAGE_CC, message->cclist);
  record_set_single_field (record, MESSAGE_BCC, message->bcclist);
  record_set_single_field (record, MESSAGE_ATTCH, message->attachlist);
  record_set_single_field (record, MESSAGE_FROM, message->from);

  record_set_single_field (record, MESSAGE_BSIZE, &message->body_size);
  record_set_single_field (record, MESSAGE_SUBJECT, message->subject);

  gint32 body_size = message->body_size;
  gpointer bp = message->body;

  gint i = 0;

  while (body_size >= BODY_UNIT_LENGTH)
  {
    record_set_multiple_field (record, MESSAGE_BODY, i++, bp);
    bp += BODY_UNIT_LENGTH;
    body_size -= BODY_UNIT_LENGTH;
  }

  if (body_size > 0)
  {
    memcpy (body_last, bp, body_size);
    record_set_multiple_field (record, MESSAGE_BODY, i, body_last);

  }

  return 0;
}


int
_record_from_match_message (msg_message_t *message, Record *record)
{
  uint32_t type = message->msg_type;
  uint32_t status = message->status;

  guchar body_last[BODY_UNIT_LENGTH];

  memset (body_last, 0x00, BODY_UNIT_LENGTH);

  if (status)
    record_set_single_field (record, MESSAGE_STATUS, &status);

  if (message->timestamp)
    record_set_single_field (record, MESSAGE_TIME, &message->timestamp);

  if (type)
    record_set_single_field (record, MESSAGE_TYPE, &type);



  record_set_single_field (record, MESSAGE_RECIPIENTS, message->recipients);
  record_set_single_field (record, MESSAGE_CC, message->cclist);
  record_set_single_field (record, MESSAGE_BCC, message->bcclist);
  record_set_single_field (record, MESSAGE_ATTCH, message->attachlist);
  record_set_single_field (record, MESSAGE_FROM, message->from);

  record_set_single_field (record, MESSAGE_SUBJECT, message->subject);

  gint32 body_size = message->body_size;
  gpointer bp = message->body;

  gint i = 0;

  while (body_size >= BODY_UNIT_LENGTH)
  {
    record_set_multiple_field (record, MESSAGE_BODY, i++, bp);
    bp += BODY_UNIT_LENGTH;
    body_size -= BODY_UNIT_LENGTH;
  }

  if (body_size > 0)
  {
    memcpy (body_last, bp, body_size);
    record_set_multiple_field (record, MESSAGE_BODY, i, body_last);

  }

  return 0;
}


int
_update_msg_refno (msg_msgid_t msgid, uint32_t ref_no)
{

  gint ret = 0;

  FieldTemplate *template = NULL;

  RecordDB *database = NULL;
  Record *record = NULL;
  gint updates[] = { MESSAGE_REPORT };

  gchar *msg_tmpl = _get_msg_tmpl_location_alloc ();

  if (!msg_tmpl)
    return -1;

  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    g_free (msg_tmpl);
    return -1;
  }

  template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    ret = -1;
    goto end;
  }

  record = record_new (template);
  if (!record)
  {
    ret = -1;
    goto end;
  }

  record_set_single_field (record, MESSAGE_ID, &msgid);
  record_set_single_field (record, MESSAGE_REPORT, &ref_no);


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = -1;
    goto end;
  }

  if (record_db_update_record
      (database, record, sizeof (updates) / sizeof (gint), updates) != 0)
    ret = -1;

end:
  g_free (db_file);
  g_free (msg_tmpl);
  record_free (record);
  record_db_close (database);
  record_db_free (database);
  field_template_free (template);

  return ret;

}

msg_err_t
_update_msg_status (msg_msgid_t id, msg_status_t status)
{

  gint ret = 0;

  FieldTemplate *template = NULL;

  RecordDB *database = NULL;
  Record *record = NULL;
  gint updates[] = { MESSAGE_STATUS };
  uint32_t stat = status;

  gchar *msg_tmpl = _get_msg_tmpl_location_alloc ();

  if (!msg_tmpl)
    return MSG_ERROR_DB_MANIPULATE;

  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    g_free (msg_tmpl);
    return MSG_ERROR_DB_MANIPULATE;
  }

  template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }

  record = record_new (template);
  if (!record)
  {
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }

  record_set_single_field (record, MESSAGE_ID, &id);
  record_set_single_field (record, MESSAGE_STATUS, &stat);


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = MSG_ERROR_DB_MANIPULATE;
    goto end;
  }

  if (record_db_update_record
      (database, record, sizeof (updates) / sizeof (gint), updates) != 0)
    ret = MSG_ERROR_DB_MANIPULATE;

end:
  g_free (db_file);
  g_free (msg_tmpl);
  record_free (record);
  record_db_close (database);
  record_db_free (database);
  field_template_free (template);

  return ret;
}

