/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#ifndef __MESSAGE_H
#define __MESSAGE_H

#include <record.h>
#include <database.h>

#include "msgmessage.h"
#include "msg-tmpl.h"


#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  typedef uint32_t msg_sys_flag_t;

  struct _msg_message_t
  {
    msg_msgid_t id;
    msg_msgsid_t sid;
    msg_status_t status;
    msg_time_t timestamp;
    msg_type_t msg_type;
    msg_sys_flag_t sys_flag;
    msg_size_t msg_size;
    msg_mem_t mem_type;

    char *recipients;
    char *cclist;
    char *bcclist;
    char *attachlist;
    char *subject;
    char *from;

    msg_size_t body_size;
    void *body;
  };

  gint _init_message (msg_message_t *msg);

  msg_message_t *_new_msg_by_id (msg_sid_t sid, msg_msgid_t id, char *db_file);


  void *_get_body_by_id (msg_sid_t sid, msg_msgid_t id, int *body_size,
                         gchar *db_file);
  gint _send_message (msg_message_t *message);
  gint _store_message (msg_message_t *message, msg_fid_t fid);

  gint _record_from_message (msg_message_t *message, Record *record);
  void *_get_body_from_record (Record *record, int *body_size);
  gint _delete_message (msg_msgid_t msgid, msg_fid_t fid);
  int _record_from_match_message (msg_message_t *message, Record *record);
  int _update_msg_refno (msg_msgid_t msgid, uint32_t ref_no);
  msg_err_t _update_msg_status (msg_msgid_t id, msg_status_t status);


#ifdef __cplusplus

}
#endif                          /* __cplusplus */

#endif                          /* __MESSAGE_H */

