/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <string.h>

#include "folder-opt.h"
#include "message-opt.h"
#include "id-generate.h"
#include "print-err.h"
#include "folder-tmpl.h"
#include "conffile.h"
#include "msgtypes.h"
#include "msgservice.h"

msg_err_t
msg_policy_discover (msg_sid_t sid, uint32_t *number,
                     msg_disp_policy_t **policy)
{
  return _policy_discover (sid, number, policy);
}



msg_folder_t *
msg_folder_new (msg_sid_t sid, msg_fid_t parentid)
{
  msg_folder_t *folder = NULL;

  folder = (msg_folder_t *) g_malloc0 (sizeof (msg_folder_t));
  if (!folder)
  {
    print_err ("cannot allocate memory");
    return NULL;
  }

  folder->id = _id_generate (PREFIX_FOLDER);

  folder->label = NULL;

  folder->parent_id = parentid;

  folder->children_num = 0;

  folder->folder_flag = MSG_FOLDER_USER;

  folder->type = 0;

  folder->currentsize = 0;

  folder->maxbytes = FOLDER_MAX_SIZE;

  return folder;

}


msg_err_t
msg_folder_release (msg_sid_t sid, msg_folder_t *folder)
{
  if (!folder)
    return 0;

  g_free (folder->label);

  memset (folder, 0x00, sizeof (msg_folder_t));

  g_free (folder);

  return 0;

}

msg_err_t
msg_folder_set_label (msg_sid_t sid, msg_folder_t *folder, char *label)
{
  if ((!folder) || (!label))
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  folder->label = g_strdup (label);
  return 0;
}

msg_err_t
msg_folder_set_maxbytes (msg_sid_t sid, msg_folder_t *folder,
                         uint32_t maxbytes)
{
  if (!folder)
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  folder->maxbytes = maxbytes;

  return 0;

}

msg_err_t
msg_folder_set_msg_type (msg_sid_t sid, msg_folder_t *folder, msg_type_t type)
{
  if (!folder)
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  folder->type = type;

  return 0;
}

msg_fid_t
msg_folder_get_id (msg_sid_t sid, msg_folder_t *folder)
{
  if (!folder)
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }


  return folder->id;

}

const char *
msg_folder_get_label (msg_sid_t sid, msg_folder_t *folder)
{

  if (!folder)
  {
    print_err ("invalid arguments");
    return NULL;
  }

  return folder->label;
}

msg_fid_t
msg_folder_get_parent (msg_sid_t sid, msg_folder_t *folder)
{

  if (!folder)
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  return folder->parent_id;
}

msg_folder_type_t
msg_folder_get_flag (msg_sid_t sid, msg_folder_t *folder)
{
  if (!folder)
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  return folder->folder_flag;

}

msg_type_t
msg_folder_get_msg_type (msg_sid_t sid, msg_folder_t *folder)
{
  if (!folder)
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  return folder->type;
}

uint32_t
msg_folder_get_currentsize (msg_sid_t sid, msg_folder_t *folder)
{
  if (!folder)
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  return folder->currentsize;
}


uint32_t
msg_folder_get_maxbytes (msg_sid_t sid, msg_folder_t *folder)
{
  if (!folder)
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  return folder->maxbytes;
}

uint32_t
msg_folder_get_has_children (msg_sid_t sid, const msg_folder_t *folder)
{
  if (!folder)
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  return folder->children_num;
}

msg_err_t
msg_folder_add_child (msg_sid_t sid, msg_fid_t parentid, msg_folder_t *folder)
{
  if (!folder)
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  folder->parent_id = parentid;

  if (_add_folder (parentid, folder) != 0)
    return MSG_ERROR_DB_MANIPULATE;
  else
    return 0;
}


msg_err_t
msg_folder_remove_child (msg_sid_t sid, msg_fid_t parentid, msg_fid_t childid)
{
  gint ret = 0;


  if (!_is_folder_empty (childid))
  {
    print_err ("folder is not empty");
    return MSG_ERROR_FOLDER_NOT_EMPTY;
  }

  ret = _remove_folder (parentid, childid);

  if (ret == -3)
  {
    print_err ("delete folder from wrong parent");
    return MSG_ERROR_WRONG_PARENT;
  }

  else if (ret == -1)
  {
    print_err ("database manipulate. parent folder is unchanged");
    return MSG_ERROR_DB_MANIPULATE;
  }

  else if (ret == -2)
  {
    if (_add_childnum (parentid, 1) == 0)
      print_err ("database manipulate. parent folder is unchanged");
    else
      print_err ("database manipulate. parent folder is changed");

    return MSG_ERROR_DB_MANIPULATE;
  }

  return 0;

}


msg_err_t
msg_folder_list_children (msg_sid_t sid, msg_fid_t folder_id,
                          uint32_t *number, msg_folder_t ***folderarray)
{

  return _list_children_folder (folder_id, number, folderarray);

}


msg_err_t
msg_folder_add_msg (msg_sid_t sid, msg_fid_t fid, msg_message_t *message)
{
  gint ret = 0;

  ret = _folder_add_size (fid, 1);
  if (ret == -2)
  {
    print_err ("parent folder is out range");
    return MSG_ERROR_FOLDER_OUT_RANGE;
  }

  else if (ret == -1)
  {
    print_err ("error on adding message to parent folder");
    return MSG_ERROR_ADD_MSG_TO_FLD;
  }
#ifdef MSGDEBUG
  g_print ("%s: folder size added\n", __FUNCTION__);
#endif

  message->timestamp = time (NULL);

  if (_store_message (message, fid) != 0)
  {
    if (_folder_add_size (fid, -1) == 0)
      print_err ("message store. the parent folder size is unchanged");
    else
      print_err ("message store. the parent folder size is changed");

    return MSG_ERROR_STORE_MESSAGE;
  }

#ifdef MSGDEBUG
  g_print ("%s: message has been stored\n", __FUNCTION__);
#endif

  return 0;

}

msg_err_t
msg_folder_remove_msg (msg_sid_t sid, msg_fid_t fid, msg_msgid_t id)
{
  gint ret = 0;

  ret = _delete_message (id, fid);
  if (ret == -1)
  {
    print_err ("removing message");
    return MSG_ERROR_DELETE_MESSAGE;
  }

  if (ret == -3)
    return msg_service_msg_delete (id);


  if (ret == -2)
  {
    print_err ("delete message from wrong folder");
    return MSG_ERROR_WRONG_PARENT;
  }

  else if (_folder_add_size (fid, -1) != 0)
  {
    print_err ("deleting message from folder");
    return MSG_ERROR_DEL_MSG_FROM_FLD;
  }

  return 0;

}

msg_err_t
msg_folder_move_msg (msg_sid_t sid, msg_fid_t from, msg_fid_t to,
                     msg_msgid_t msgid)
{
  msg_err_t ret = 0;

  ret = _change_message_parent (from, to, msgid);
  if (ret != 0)
    return ret;


  if (_folder_add_size (to, 1) != 0)
  {
    if (_change_message_parent (to, from, msgid) == 0)
      print_err ("adding message to folder. message parent is unchanged");
    else
      print_err ("adding message to folder. message parent is changed");
    return MSG_ERROR_ADD_MSG_TO_FLD;
  }

  if (_folder_add_size (from, -1) != 0)
  {
    _folder_add_size (to, -1);
    _change_message_parent (to, from, msgid);
    print_err
      ("deleting message from parent. the status of folder and message is unknown");
    return MSG_ERROR_DEL_MSG_FROM_FLD;
  }

  return 0;

}

uint32_t
msg_folder_count_msgs (msg_sid_t sid, msg_fid_t fid, msg_status_t status,
                       bool recursive)
{
  uint32_t dev_num = 0;
  uint32_t sim_num = 0;
  uint32_t ret = 0;


  char *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    return -1;
  }


  char *tmp_db = _get_tmpdb_location_alloc ();

  if (!tmp_db)
  {
    g_free (tmp_db);
    return -1;
  }

  if (recursive)
    dev_num = _count_msg_in_folder_r (fid, status);
  else
    dev_num = _count_msg_in_folder (fid, status, db_file);

  if (dev_num == -1)
  {
    ret = -1;
    goto end;
  }

  if (g_file_test (tmp_db, G_FILE_TEST_EXISTS))
  {
    sim_num = _count_msg_in_folder (fid, status, tmp_db);
    if (sim_num == -1)
    {
      ret = -1;
      goto end;
    }
  }


  ret = sim_num + dev_num;

end:
  g_free (db_file);
  g_free (tmp_db);
  return ret;
}

msg_err_t
msg_folder_list_msg_dig (msg_sid_t sid, msg_fid_t fid, uint32_t *number,
                         msg_message_t ***msgarray)
{
  msg_err_t ret = 0;
  msg_message_t **msgarray_se = NULL;
  uint32_t num_se = 0;
  uint32_t num_sim = 0;
  msg_message_t **msgarray_sim = NULL;
  int i = 0;

  char *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    return MSG_ERROR_READ_CONF_FILE;
  }

  char *tmp_db = _get_tmpdb_location_alloc ();

  if (!tmp_db)
  {
    g_free (db_file);
    return MSG_ERROR_READ_CONF_FILE;
  }

  ret = _list_message_dig (fid, &num_se, &msgarray_se, db_file, MSG_MEM_ME);
  if (ret != 0)
    goto end;

  if (g_file_test (tmp_db, G_FILE_TEST_EXISTS))
  {
    ret = _list_message_dig (fid, &num_sim, &msgarray_sim, tmp_db, MSG_MEM_SIM);
    if (ret != 0)
    {
      for (i = 0; i < num_se; i++)
        msg_release (sid, msgarray_se[i]);
      goto end;
    }
  }

  *msgarray = g_new0 (msg_message_t *, num_se + num_sim);
  if (!*msgarray)
  {
    for (i = 0; i < num_se; i++)
      msg_release (sid, msgarray_se[i]);
    for (i = 0; i < num_sim; i++)
      msg_release (sid, msgarray_sim[i]);
    ret = MSG_ERROR_MEM_ALLOC;
    goto end;
  }

  for (i = 0; i < num_se; i++)
    (*msgarray)[i] = msgarray_se[i];
  for (i = 0; i < num_sim; i++)
    (*msgarray)[i + num_se] = msgarray_sim[i];

  *number = num_se + num_sim;


end:
  g_free (tmp_db);
  g_free (db_file);
  g_free (msgarray_se);
  g_free (msgarray_sim);
  return ret;

}

/** currently in following function, the seach field cannot contain body field
 and return argument msgarray contain body information
 */
msg_err_t
msg_folder_search_msgs (msg_sid_t sid, msg_fid_t fid, msg_message_t *match,
                        uint32_t *number, msg_message_t ***msgarray)
{


  msg_err_t ret = 0;
  msg_message_t **msgarray_se = NULL;
  uint32_t num_se = 0;
  uint32_t num_sim = 0;
  msg_message_t **msgarray_sim = NULL;
  int i = 0;

  char *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    return MSG_ERROR_READ_CONF_FILE;
  }

  char *tmp_db = _get_tmpdb_location_alloc ();

  if (!tmp_db)
  {
    g_free (db_file);
    return MSG_ERROR_READ_CONF_FILE;
  }

  ret =
    _message_search (fid, match, &num_se, &msgarray_se, db_file, MSG_MEM_ME);
  if (ret != 0)
    goto end;

  if (g_file_test (tmp_db, G_FILE_TEST_EXISTS))
  {
    ret =
      _message_search (fid, match, &num_sim, &msgarray_sim, tmp_db,
                       MSG_MEM_SIM);
    if (ret != 0)
    {
      for (i = 0; i < num_se; i++)
        msg_release (sid, msgarray_se[i]);
      goto end;
    }
  }


  *msgarray = g_new0 (msg_message_t *, num_se + num_sim);
  if (!*msgarray)
  {
    for (i = 0; i < num_se; i++)
      msg_release (sid, msgarray_se[i]);
    for (i = 0; i < num_sim; i++)
      msg_release (sid, msgarray_sim[i]);
    ret = MSG_ERROR_MEM_ALLOC;
    goto end;
  }

  for (i = 0; i < num_se; i++)
    (*msgarray)[i] = msgarray_se[i];
  for (i = 0; i < num_sim; i++)
    (*msgarray)[i + num_se] = msgarray_sim[i];

  *number = num_se + num_sim;


end:
  g_free (tmp_db);
  g_free (db_file);
  g_free (msgarray_se);
  g_free (msgarray_sim);
  return ret;

}

