/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include <string.h>

#include "msgfolder.h"
#include "message-opt.h"
#include "print-err.h"
#include "conffile.h"


msg_message_t *
msg_new (msg_sid_t sid, msg_type_t msg_type)
{

  msg_message_t *msg = NULL;

  msg = (msg_message_t *) g_malloc0 (sizeof (msg_message_t));
  if (!msg)
  {
    print_err ("memory allocation");
    return NULL;
  }

  _init_message (msg);

  msg->sid = sid;
  msg->msg_type = msg_type;

  return msg;
}


msg_err_t
msg_release (msg_sid_t sid, msg_message_t *message)
{
  message->id = 0;
  message->sid = 0;
  message->status = 0;
  message->timestamp = 0;
  message->msg_type = 0;
  message->sys_flag = 0;
  message->msg_size = 0;



  g_free (message->recipients);
  g_free (message->cclist);
  g_free (message->bcclist);
  g_free (message->attachlist);
  g_free (message->subject);
  g_free (message->from);
  g_free (message->body);

  g_free (message);

  return 0;

}


msg_err_t
msg_set_recipientlist (msg_sid_t sid, msg_message_t *message,
                       char *recipientlist)
{
  if ((!message) || (!recipientlist))
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  message->recipients = g_strdup (recipientlist);
  message->msg_size += strlen (recipientlist);

  return 0;

}

msg_err_t
msg_set_cclist (msg_sid_t sid, msg_message_t *message, char *cclist)
{

  if ((!message) || (!cclist))
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  message->cclist = g_strdup (cclist);
  message->msg_size += strlen (cclist);

  return 0;
}

msg_err_t
msg_set_bcclist (msg_sid_t sid, msg_message_t *message, char *bcclist)
{

  if ((!message) || (!bcclist))
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  message->bcclist = g_strdup (bcclist);
  message->msg_size += strlen (bcclist);

  return 0;

}

msg_err_t
msg_set_attachlist (msg_sid_t sid, msg_message_t *message, char *attachlist)
{

  if ((!message) || (!attachlist))
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  message->attachlist = g_strdup (attachlist);
  message->msg_size += strlen (attachlist);

  return 0;
}

msg_err_t
msg_set_subject (msg_sid_t sid, msg_message_t *message, char *subject)
{

  if ((!message) || (!subject))
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  message->subject = g_strdup (subject);
  message->msg_size += strlen (subject);

  return 0;
}


msg_err_t
msg_set_timestamp (msg_sid_t sid, msg_message_t *message, msg_time_t timestamp)
{

  if ((!message))
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  message->timestamp = timestamp;

  return 0;
}

msg_err_t
msg_set_from (msg_sid_t sid, msg_message_t *message, char *from)
{

  if ((!message) || (!from))
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  message->from = g_strdup (from);
  message->msg_size += strlen (from);

  return 0;
}


msg_err_t
msg_set_status (msg_sid_t sid, msg_message_t *message, msg_status_t status)
{
  if (!message)
    return MSG_ERROR_INVALID_ARG;

  message->status = status;
  return 0;
}

msg_err_t
msg_set_body (msg_sid_t sid, msg_message_t *message, void *body, int body_size)
{

  if (!message)
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }

  if (body_size == 0)
    return 0;

  if (!body)
  {
    print_err ("invalid arguments");
    return MSG_ERROR_INVALID_ARG;
  }


  message->body = g_malloc0 (body_size);

  if (!message->body)
  {
    print_err ("cannot allocate memory");
    return MSG_ERROR_MEM_ALLOC;
  }

  g_memmove (message->body, body, body_size);
  message->body_size = body_size;
  message->msg_size += body_size;

  return 0;

}


const char *
msg_get_recipientlist (msg_sid_t sid, msg_message_t *message)
{

  if (!message)
  {
    print_err ("invalid arguments");
    return NULL;
  }

  return message->recipients;
}

const char *
msg_get_cclist (msg_sid_t sid, msg_message_t *message)
{

  if (!message)
  {
    print_err ("invalid arguments");
    return NULL;
  }

  return message->cclist;
}

const char *
msg_get_bcclist (msg_sid_t sid, msg_message_t *message)
{

  if (!message)
  {
    print_err ("invalid arguments");
    return NULL;
  }
  return message->bcclist;
}

const char *
msg_get_attachlist (msg_sid_t sid, msg_message_t *message)
{
  if (!message)
  {
    print_err ("invalid arguments");
    return NULL;
  }
  return message->attachlist;
}

const char *
msg_get_subject (msg_sid_t sid, msg_message_t *message)
{
  if (!message)
  {
    print_err ("invalid arguments");
    return NULL;
  }

  return message->subject;
}

const char *
msg_get_from (msg_sid_t sid, msg_message_t *message)
{
  if (!message)
  {
    print_err ("invalid arguments");
    return NULL;
  }

  return message->from;
}

msg_msgid_t
msg_get_id (msg_sid_t sid, msg_message_t *message)
{
  if (!message)
  {
    print_err ("invalid arguments");
    return -1;
  }

  return message->id;
}

msg_msgsid_t
msg_get_sid (msg_sid_t sid, msg_message_t *message)
{
  if (!message)
  {
    print_err ("invalid arguments");
    return -1;
  }

  return message->sid;
}

msg_status_t
msg_get_status (msg_sid_t sid, msg_message_t *message)
{
  if (!message)
  {
    print_err ("invalid arguments");
    return -1;
  }

  return message->status;
}


const void *
msg_get_body (msg_sid_t sid, msg_message_t *message, uint32_t *size)
{
  if (!message)
    return NULL;

  *size = message->body_size;

  return message->body;
}

msg_time_t
msg_get_timestamp (msg_sid_t sid, msg_message_t *message)
{

  if (!message)
  {
    print_err ("invalid arguments");
    return -1;
  }
  return message->timestamp;

}

msg_type_t
msg_get_type (msg_sid_t sid, msg_message_t *message)
{

  if (!message)
  {
    print_err ("invalid arguments");
    return -1;
  }

  return message->msg_type;
}

msg_size_t
msg_get_size (msg_sid_t sid, msg_message_t *message)
{
  if (!message)
  {
    print_err ("invalid arguments");
    return -1;
  }

  return message->msg_size;
}


/** getting msg body by msg identifier
*/

void *
msg_get_body_by_id (msg_sid_t sid, msg_msgid_t id, int *body_size)
{
  void *ret = NULL;

  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    return NULL;
  }

  ret = _get_body_by_id (sid, id, body_size, db_file);
  g_free (db_file);
  if (ret)
    return ret;

  db_file = _get_tmpdb_location_alloc ();
  if (!db_file)
    return NULL;

  ret = _get_body_by_id (sid, id, body_size, db_file);
  g_free (db_file);

  return ret;

}

msg_message_t *
msg_test_get_msg_by_id (msg_sid_t sid, msg_msgid_t id)
{

  msg_message_t *ret = NULL;

  gchar *db_file = _get_dbfile_location_alloc ();

  if (!db_file)
  {
    return NULL;
  }

  ret = _new_msg_by_id (sid, id, db_file);
  g_free (db_file);
  if (ret)
    return ret;

  db_file = _get_tmpdb_location_alloc ();
  if (!db_file)
    return NULL;

  ret = _new_msg_by_id (sid, id, db_file);
  g_free (db_file);

  return ret;
}


/** send msg out and almost return immediately. 
	*following response will be returned back through events.
*/

msg_err_t
msg_send (msg_sid_t sid, msg_message_t *message)
{

  if (_send_message (message) != 0)
  {
    print_err ("message sending");
    return MSG_ERROR_SEND_MESSAGE;
  }

  if (message->sys_flag)
    return 0;

  message->status = MSG_SENDING;
  message->timestamp = time (NULL);




  if (msg_folder_add_msg (sid, MSG_FOLDER_OUTBOX, message) != 0)
    return MSG_ERROR_STORE_MESSAGE;

  return 0;

}


msg_err_t
msg_update_status (msg_sid_t sid, msg_msgid_t id, msg_status_t status)
{
  return _update_msg_status (id, status);
}

