/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 * @file API definition of the LiPS Message Service interface - Message
 *
 * Abbreviation:
 * LiPS: Linux Phone Standard
 */

#ifndef __LIPS_MSG_MESSAGE_H
#define __LIPS_MSG_MESSAGE_H


#include <msgtypes.h>
#include <msgerror.h>

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */




/******************** message manipulation functions****************** 
 */


/** 
 *@brief: create a new message with type msg_message_type
*/
  msg_message_t *msg_new (msg_sid_t sid, msg_type_t msg_type);

/** 
 *@brief:free the msg structure
*/
  msg_err_t msg_release (msg_sid_t sid, msg_message_t *message);


/** 
 *@brief:setting header fields of msg. 
 */
  msg_err_t msg_set_recipientlist (msg_sid_t sid, msg_message_t *message,
                                   char *recipientlist);

  msg_err_t msg_set_cclist (msg_sid_t sid, msg_message_t *message,
                            char *cclist);

  msg_err_t msg_set_bcclist (msg_sid_t sid, msg_message_t *message,
                             char *bcclist);

  msg_err_t msg_set_attachlist (msg_sid_t sid, msg_message_t *message,
                                char *attachlist);

  msg_err_t msg_set_subject (msg_sid_t sid, msg_message_t *message,
                             char *subject);

  msg_err_t msg_set_from (msg_sid_t sid, msg_message_t *message, char *from);

  msg_err_t msg_set_status (msg_sid_t sid, msg_message_t *message,
                            msg_status_t status);

  msg_err_t msg_set_timestamp (msg_sid_t sid, msg_message_t *message,
                               msg_time_t timestamp);
/** 
 *@brief:setting body field of msg. 
 *app will take the responsibility composing body beforehand.
 *As to SMS app, body field will be raw message
*/
  msg_err_t msg_set_body (msg_sid_t sid, msg_message_t *message, void *body,
                          int body_size);

/** a group of getting functions
*/
  const char *msg_get_recipientlist (msg_sid_t sid, msg_message_t *message);
  const char *msg_get_cclist (msg_sid_t sid, msg_message_t *message);
  const char *msg_get_bcclist (msg_sid_t sid, msg_message_t *message);
  const char *msg_get_attachlist (msg_sid_t sid, msg_message_t *message);
  const char *msg_get_subject (msg_sid_t sid, msg_message_t *message);
  const char *msg_get_from (msg_sid_t sid, msg_message_t *message);
  const void *msg_get_body (msg_sid_t sid, msg_message_t *message,
                            uint32_t *size);



  msg_msgid_t msg_get_id (msg_sid_t sid, msg_message_t *message);
  msg_msgsid_t msg_get_sid (msg_sid_t sid, msg_message_t *message);
  msg_status_t msg_get_status (msg_sid_t sid, msg_message_t *message);
  msg_time_t msg_get_timestamp (msg_sid_t sid, msg_message_t *message);
  msg_type_t msg_get_type (msg_sid_t sid, msg_message_t *message);
  msg_size_t msg_get_size (msg_sid_t sid, msg_message_t *message);


/** getting msg body by msg identifier
*/
  void *msg_get_body_by_id (msg_sid_t sid, msg_msgid_t id, int *body_size);

  msg_message_t *msg_test_get_msg_by_id (msg_sid_t sid, msg_msgid_t id);


/** send msg out and almost return immediately. 
  *following response will be returned back through events.
*/
  msg_err_t msg_send (msg_sid_t sid, msg_message_t *message);

  msg_err_t msg_update_status (msg_sid_t sid, msg_msgid_t id,
                               msg_status_t status);

/**msg get is out of scope. when there is some new msgs available, 
  *they will be stored into DB internally and then an MSG_INCOMING 
  *event is passed to applications .
*/


#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __LIPS_MSG_MESSAGE_H */

