/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include <glib.h>
#include <string.h>
#include <stdio.h>

#include <msgerror.h>
#include <msgtypes.h>

static GHashTable *session_table;
static int curr_index;

typedef struct
{
  char *ses;
} _msg_session_t;

void
_session_free (gpointer data)
{
  if (!data)
    return;

  _msg_session_t *value = (_msg_session_t *) data;

  g_free (value->ses);
  g_free (value);

  return;
}

void _session_init (_msg_session_t *session);

char *_get_session (msg_sid_t sid);

msg_err_t
msg_open_session (msg_sid_t * sidp)
{
  int *key = (int *) g_malloc0 (sizeof (int));
  _msg_session_t *value = NULL;

  if (!session_table)
  {
    session_table = g_hash_table_new_full (g_int_hash, g_int_equal, g_free,
                                           _session_free);
    if (!session_table)
      return MSG_ERROR_HASH_TABLE;
    curr_index = 0;
  }

  value = (_msg_session_t *) g_malloc0 (sizeof (_msg_session_t));
  if (!value)
    return MSG_ERROR_MEM_ALLOC;

  _session_init (value);

  *key = ++curr_index;

  g_hash_table_insert (session_table, key, value);

  value = NULL;

  value = g_hash_table_lookup (session_table, key);


  *sidp = *key;

  return MSG_ERROR_NONE;

}


msg_err_t
msg_close_session (msg_sid_t sid)
{

  if (!session_table)
    return MSG_ERROR_HASH_TABLE;

  if (!g_hash_table_lookup (session_table, &sid))
    return MSG_ERROR_NONE;

  if (!g_hash_table_remove (session_table, &sid))
    return MSG_ERROR_HASH_TABLE;

  if (g_hash_table_size (session_table) == 0)
    g_hash_table_destroy (session_table);

  return MSG_ERROR_NONE;

}

char *
_get_session (msg_sid_t sid)
{
  _msg_session_t *session = NULL;

  if (!session_table)
    return NULL;

  printf ("table size is %d\n", g_hash_table_size (session_table));

  session = g_hash_table_lookup (session_table, &sid);

  return session->ses;
}


void
_session_init (_msg_session_t *session)
{
  session->ses = strdup ("session is open");
  return;
}

void
session_table_free ()
{

  if (session_table)
  {
#ifdef	MSGDEBUG
    g_print ("size is %d\n", g_hash_table_size (session_table));
#endif
    g_hash_table_destroy (session_table);
  }
  return;
}

void
session_table_init ()
{

  if (!session_table)
  {
    session_table = g_hash_table_new_full (g_int_hash, g_int_equal, g_free,
                                           _session_free);
    curr_index = 0;
  }

  return;
}

