/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdint.h>
#include <stdlib.h>


#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

#define MAX_NUMBER_LEN 40
#define MAX_SMS_LEN 160
#define MAX_SEGMENT_NUMBER 3

/** message type
*/
  typedef enum
  {
    SMS_SUBMIT,                         /**< MO SMS , from mobile phone*/
    SMS_DELIVER,                /**< MT SMS , from network*/
    SMS_STATUS_REPORT,          /**< SMS Status report*/
    SMS_COMMAND                         /**< SMS command*/
  } sms_type;

  typedef enum
  {
    SMS_REC_UNREAD,
    SMS_REC_READ,
    SMS_STO_UNSENT,
    SMS_STO_SENT
  } sms_status;

  typedef enum
  {
    SMSMEMTYPE_SIM,                                             /**< SIM memory */
    SMSMEMTYPE_ME,                                              /**< ME's memory */
  } sms_mem_type;


  typedef time_t sms_timestamp_t;

  typedef enum
  {
    UnknownTON = 0,
    InternationalTON = 1,
    NationalTON = 2,
    NetworkSpecificTON = 3,
    DedicatedAccessTON = 4,
    DefaultAlphabetTON = 5
  } sms_number_TON;

/** This holds a number with TON and number
*/
  typedef struct
  {
    sms_number_TON type;
    char number[MAX_NUMBER_LEN];
  } sms_number_t;

  typedef struct
  {
    uint8_t text[MAX_SMS_LEN];  /* Maximum length for a SMS. */
    int8_t curr;                /* Current SMS segment Index in concatenated message */
    int8_t total;               /* Total SMS segments in concatenated message */
  } sms_userdata_t;

  typedef struct
  {
/* General fields */
    sms_type type;              /* Type of the message. valid - MO, MT */
    sms_status status;          /* Status of the message. valid - MO, MT */
    sms_mem_type memory_type;   /* Memory type where the message is/should be stored. valid - MO, MT */


/* Status report related fields */
    uint8_t ref_no;             /* the reference number. valid - STATUS Report */
    uint8_t delevery_status;    /* the message delivery status. valid - STATUS Report */
    uint32_t serial_no;

/* time stamp fields */
    sms_timestamp_t timestamp;  /* Delivery timestamp. valid - MT */

/* Number related fields */
    sms_number_t remote;        /* Remote (sender/receipient) number. valid - MO, MT */


/* Date fields */
    int8_t curr;
    int8_t total;
    char *body;

  } trans_sms_t;

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

