/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <database.h>
#include "message-opt.h"
#include "folder-opt.h"
#include "conffile.h"
#include "policy-tmpl.h"
#include "folder-tmpl.h"
#include "print-err.h"

int
_tmpdb_store_message_array (msg_message_t **msg_a, uint32_t *index_a,
                            int size)
{
  gint ret = 0;

  FieldTemplate *template = NULL;

  RecordDB *database = NULL;
  Record *record = NULL;
  int i = 0;
  uint32_t fid = 0;

  gchar *msg_tmpl = _get_msg_tmpl_location_alloc ();

  if (!msg_tmpl)
    return -1;

  gchar *db_file = _get_tmpdb_location_alloc ();

  if (!db_file)
  {
    g_free (msg_tmpl);
    return -1;
  }

  template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    ret = -1;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = -1;
    goto end;
  }

  for (i = 0; i < size; i++)
  {

    record = record_new (template);
    if (!record)
    {
      ret = -1;
      goto end;
    }

    _record_from_message (msg_a[i], record);

    if ((msg_a[i]->status == MSG_RECEIVED_UNREAD)
        || (msg_a[i]->status == MSG_RECEIVED_READ))
      fid = MSG_FOLDER_INBOX;
    else if (msg_a[i]->status == MSG_SENT_FAILED)
      fid = MSG_FOLDER_OUTBOX;
    else
      fid = MSG_FOLDER_SENT;

    record_set_single_field (record, MESSAGE_PARENT, &fid);

    record_set_single_field (record, MESSAGE_REPORT, &index_a[i]);
    /* use MESSAGE_REPORT fields to store index num temperarily. */

    if (record_db_add_record (database, record) != 0)
    {
      ret = -1;
      goto end;
    }

    record_free (record);

  }


end:
  g_free (db_file);
  g_free (msg_tmpl);
  record_db_close (database);
  record_db_free (database);
  field_template_free (template);

  return ret;
}


int
_tmpdb_store_message (msg_message_t *message)
{
  gint ret = 0;

  FieldTemplate *template = NULL;

  RecordDB *database = NULL;
  Record *record = NULL;
  uint32_t fid = 0;

  gchar *msg_tmpl = _get_msg_tmpl_location_alloc ();

  if (!msg_tmpl)
    return -1;

  gchar *db_file = _get_tmpdb_location_alloc ();

  if (!db_file)
  {
    g_free (msg_tmpl);
    return -1;
  }

  template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    ret = -1;
    goto end;
  }

  record = record_new (template);
  if (!record)
  {
    ret = -1;
    goto end;
  }

  _record_from_message (message, record);


  if ((message->status == MSG_RECEIVED_UNREAD)
      || (message->status == MSG_RECEIVED_READ))
    fid = MSG_FOLDER_INBOX;
  else if (message->status == MSG_SENT_FAILED)
    fid = MSG_FOLDER_OUTBOX;
  else
    fid = MSG_FOLDER_SENT;

  record_set_single_field (record, MESSAGE_PARENT, &fid);
  printf ("store fid is %d\n", fid);

  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = -1;
    goto end;
  }

  if (record_db_add_record (database, record) != 0)
    ret = -1;

end:
  g_free (msg_tmpl);
  record_free (record);
  g_free (db_file);
  record_db_close (database);
  record_db_free (database);
  field_template_free (template);

  return ret;
}

int
_tmpdb_delete_message (msg_msgid_t msgid)
{
  gint ret = 0;

  FieldTemplate *template = NULL;
  Record *record = NULL;
  RecordDB *database = NULL;

  gchar *msg_tmpl = _get_msg_tmpl_location_alloc ();

  if (!msg_tmpl)
    return -1;

  gchar *db_file = _get_tmpdb_location_alloc ();

  if (!db_file)
  {
    g_free (msg_tmpl);
    return -1;
  }

  template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    ret = -1;
    goto end;
  }

  record = record_new (template);
  if (!record)
  {
    ret = -1;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = -1;
    goto end;
  }


  if (record_db_remove_record (database, &msgid) != 0)
    ret = -1;

end:
  g_free (db_file);
  g_free (msg_tmpl);
  record_db_close (database);
  record_db_free (database);
  field_template_free (template);

  return ret;
}

uint8_t
_tdb_find_rec_no (msg_msgid_t msgid)
{

  uint8_t ret = 0;

  FieldTemplate *template = NULL;
  Record *record = NULL;
  RecordDB *database = NULL;

  gchar *msg_tmpl = _get_msg_tmpl_location_alloc ();

  if (!msg_tmpl)
    return -1;

  gchar *db_file = _get_tmpdb_location_alloc ();

  if (!db_file)
  {
    g_free (msg_tmpl);
    return -1;
  }

  template = field_template_new ();

  if (field_template_parse_file (template, msg_tmpl) != TRUE)
  {
    ret = -1;
    goto end;
  }

  record = record_new (template);
  if (!record)
  {
    ret = -1;
    goto end;
  }


  database = record_db_new (db_file, PREFIX_MESSAGE, template);
  if (record_db_open (database) != DB_ERROR_NONE)
  {
    ret = -1;
    goto end;
  }

  record_db_get_record (database, record, &msgid);

  ret = *(uint32_t *) record_get_single_field_static (record, MESSAGE_REPORT);

end:
  g_free (db_file);
  g_free (msg_tmpl);
  record_db_close (database);
  record_db_free (database);
  field_template_free (template);

  return ret;
}

