/*	libmsgenabler - Library of messaging enabler
 *
 *	Authors: WU Yu <yu.wu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <string.h>

#include "message-opt.h"
#include "sms.h"
#include "id-generate.h"


msg_message_t *
_new_message_from_trans_data (trans_sms_t *trans_data)
{
  msg_message_t *msg = (msg_message_t *) g_malloc0 (sizeof (msg_message_t));

  msg->id = _id_generate (PREFIX_MESSAGE);
  g_print ("%s: id generated\n", __FUNCTION__);


  switch (trans_data->status)
  {
    case SMS_REC_UNREAD:
      msg->status = MSG_RECEIVED_UNREAD;
      break;

    case SMS_REC_READ:
      msg->status = MSG_RECEIVED_READ;
      break;

    case SMS_STO_UNSENT:
      msg->status = MSG_SENT_FAILED;
      break;

    case SMS_STO_SENT:
      msg->status = MSG_SENT;
      break;
  }

  msg->sys_flag = 0;

  /** added later:
		 msg->sid = ...
	*/

  msg->msg_type = MSG_SMS;
  msg->timestamp = trans_data->timestamp;

  msg->from = g_strdup (trans_data->remote.number);
  g_print ("%s: remote number %s\n", __FUNCTION__, msg->from);


  g_print ("%s: trans body is %s\n", __FUNCTION__, trans_data->body);

  msg->body = g_strdup (trans_data->body);

  g_print ("%s: body copied, %s\n", __FUNCTION__, (char *) msg->body);

  msg->body_size = strlen ((char *) msg->body) + 1;

  g_print ("%s: body size generated %d\n", __FUNCTION__, msg->body_size);
  msg->msg_size = msg->body_size + strlen (msg->from);
  g_print ("%s: message size is ok\n", __FUNCTION__);

  return msg;

}

int
_message_to_trans_data (msg_message_t *message, trans_sms_t *trans_data)
{

  trans_data->type = SMS_SUBMIT;
  trans_data->status = SMS_STO_UNSENT;
  trans_data->memory_type = SMSMEMTYPE_ME;

  trans_data->serial_no = message->id;

  if (!message->recipients)
    return -1;

  if (*message->recipients == '+')
  {
    trans_data->remote.type = InternationalTON;
    strcpy (trans_data->remote.number, (message->recipients + 1));
  }
  else
  {
    trans_data->remote.type = NationalTON;
    strcpy (trans_data->remote.number, message->recipients);
  }


  trans_data->body = g_strdup (message->body);


  return 0;

}

